# Fidelity-api

Api desenvolvida para permitir a utilização da rotina de fidelidade de forma online. O objetivo desta implementação é otimizar a validação de saldo do consumidor
e possibilitar implementação de novas funcionalidades na gestão das campanhas fidelidade e consumidores.

**Estrutura de projeto**

> | Configurações    |         |
> | ---------------- | :------ |
> | Server           | Linux   |
> | Versão de PHP    | PHP 7.4 |
> | Framework        | Slim V3 |
> | Tipo de Conteúdo | XML     |
> | Conexão BD       | Oracle  |
>
> ---

> | Endereços          |                                                                                 |
> | ------------------ | :------------------------------------------------------------------------------ |
> | DNS de homologação | https://marcossouza2.zeedhi.com/workfolder/fidelidade/backend/service/index.php |
> | DNS de Produção    | Indisponível                                                                    |
>
> ---

> **Tabela de tipos de campanha**
>
> ---
>
> | ID  | DESCRIÇÃO            |
> | :-: | :------------------- |
> |  E  | DESCONTO POR VENDA   |
> |  V  | DOTZ                 |
> |  P  | PRODUTO              |
> |  S  | SALDO POR % DE VENDA |
> |  D  | % POR PRODUTO        |
>
> ---

### Documentation

- [Documentation](https://documenter.getpostman.com/view/18727506/2s9YsKfWqu)

### Cabeçalho 

É necessário enviar as informações de nome, organização e tipo para a construção da conexão à base de dados do cliente. Segue exemplo:

> ---
>
> | CHAVE         | VALOR        |
> |      :-:      | :----------- |
> | x-client-name | Nome         |
> | x-client-env  | usr_org_9999 |
> | x-client-type | saas         |
>
> ---

### Rotas 

<details>
  <summary>
    <code>GET</code>
    <code><b>/</b></code>
  </summary>

##### Response

```   
<root>
  <error>false</error>
  <message></message>
  <data>
    <version>1.0.0</version>
    <revision>1</revision>
    <date>20-12-2023 09:00</date>
    <branchs>2</branchs>
  </data>
</root>
```

</details>

---

<details>
  <summary>
    <code>POST</code>
    <code><b>/getBalance</b></code>
  </summary>

##### Payload

```

    <?xml version="1.0" encoding="UTF-8"?>
        <root>
            <data>
                <clientCode>O00001</clientCode> // Código do cliente
                <consumerCode>000000000031988887777</consumerCode> // Código do consumidor
                <campaignCode>E</campaignCode> // Código da Campanha
            </data>
        </root>

```

##### Response

```
<?xml version="1.0" encoding="UTF-8"?>
    <root>
    <error>false</error>
        <message></message>
            <data>
                <consumerName>CRM BONUS</consumerName> // Nome do consumidor
                <consumerCode>000000000031999999999</consumerCode> // Código do consumidor
                <clientCode>O00001</clientCode> // Código do cliente
                <campaignCode>0000000024</campaignCode> // Código da campanha
                <balance>0.57</balance> // Saldo disponível
            </data>
    </root>
```

</details>

---

<details>
  <summary>
    <code>POST</code>
    <code><b>/creditBalance</b></code>
  </summary>

##### Payload

```
<?xml version="1.0" encoding="UTF-8"?>
    <root>
        <data>
            <subsidiaryCode>0107</subsidiaryCode> // Código da filial
            <posCode>002</posCode> // Código do caixa
            <saleSeq>0000000004</saleSeq> // Número sequencial da venda
            <clientCode>O00001</clientCode> // Código do cliente
            <consumerCode>000000000031988887777</consumerCode> // Código do consumidor
            <phone>31986834520</phone> // Telefone do Consumidor
            <posOpeningDate>10/01/2024 09:29:15</posOpeningDate>
        <products>
            <prod>
                <productCode>9010103300</productCode> // Código do produto
                <quantity>1</quantity> // Quantidade vendida
                <unitValue>5.90</unitValue> // Valor unitário
                <totalValue>5.90</totalValue> // Valor total
            </prod>
            <prod>
                <productCode>9010103300</productCode> // Código do produto
                <quantity>1</quantity> // Quantidade vendida
                <unitValue>12.50</unitValue> // Valor unitário
                <totalValue>6.90</totalValue> // Valor total
            </prod>
        </products>
        </data>
    </root>

```

##### Response

```
 <root>
    <error>false</error>
    <message></message>
    <data>
        <consumerName>NOVO_CONS</consumerName> // Nome do consumidor
        <consumerCode>000000000031988887777</consumerCode> // Código do consumidor
        <clientCode>O00001</clientCode> // Código do cliente
        <qrCode>data:image/svg+xml;base64,PHN2ZyB4bWxu...</qrCode> // QRCode da url encurtada pelo URL Shortener
        <balance>
            <campaignCode>0000000024</campaignCode> // Código da campanha
            <campaignId>S</campaignId> // ID da Campanha
            <campaignName>FIDELIDADE GIRAFFAS</campaignName> // Nome da campanha
            <generatedBalance>1.28</generatedBalance> // Saldo gerado
            <totalBalance>2.28</totalBalance> // Saldo total
            <expiration>29/01/2024</expiration> // Data de validade do saldo
        </balance>
    </data>
</root>

```

</details>

---

<details>
  <summary>
    <code>POST</code>
    <code><b>/redeemBalance</b></code>
  </summary>

##### Payload

```
<?xml version="1.0" encoding="UTF-8"?>
    <root>
        <data>
            <subsidiaryCode>0107</subsidiaryCode> // Código da filial
            <posCode>002</posCode> // Código do caixa
            <clientCode>O00001</clientCode> // Código do cliente
            <consumerCode>000000000031988887777</consumerCode> // Código do consumidor
            <phone>31986834520</phone> // Telefone
            <campaignCode>0000000024</campaignCode> // Código da campanha
            <saleSeq>0000000003</saleSeq> // Numero sequencial da venda
            <posOpeningDate>10/01/2024 09:29:15</posOpeningDate> // Horario de abertura do caixa
            <balanceUsed>1.2</balanceUsed> // Saldo utilizado
        </data>
    </root>
```

##### Response

```
<root>
    <error>false</error>
    <message>Resgate de saldo realizado com sucesso.</message>
    <data/>
</root>
```

</details>

---

<details>
  <summary>
    <code>POST</code>
    <code><b>/cancelBalance</b></code>
  </summary>

##### Payload

```
<?xml version="1.0" encoding="UTF-8"?>
    <root>
        <data>
            <subsidiaryCode>0107</subsidiaryCode> // Código da filial
            <posCode>002</posCode> // Código do caixa
            <clientCode>O00001</clientCode> // Código do cliente
            <consumerCode>000000000031988887777</consumerCode> // Código do consumidor
            <saleSeq>0000000003</saleSeq> // Número sequencial da venda
            <posOpeningDate>10/01/2024 09:29:15</posOpeningDate> // Horario de abertura do caixa
        </data>
    </root>
```

##### Response

```
{
    "root": {
        "error": "", // Status de erro
        "message": "", // Mensagem de retorno do cancelamento
        "data": ""
    }
}
```

</details>

---

<details>
  <summary>
    <code>POST</code>
    <code><b>/getConsumer</b></code>
  </summary>

##### Payload

```

<?xml version="1.0" encoding="UTF-8"?>
    <root>
        <data>
            <id>31988889999</id> // Identificação do consumidor
            <type>phone</type> // Tipo da identificação
        </data>
    </root>

```

##### Response

```
<root>
    <error>false</error>
    <message></message>
    <data>
        <clientCode>O00001</clientCode> // Código do cliente
        <consumerCode>000000000031988887777</consumerCode> // Código do consumidor
        <name>NOVO_CONS</name> // Nome do consumidor
        <email></email> // Email do consumidor
        <phone>31988889999</phone> // Telefone do consumidor
        <cpf></cpf> // CPF do consumidor
        <birthday></birthday> // Data de nascimento
        <gender></gender> // Gênero
        <cep></cep> // Cep do consumidor
        <street></street> // Rua do endereço do consumidor
            <neighborhood></neighborhood> // Bairro do endereço do consumidor
            <addressNumber></addressNumber> // Número da residencia
            <complement></complement> // Complemento do endereço
            <state></state> // Estado 
            <city></city> // Cidade
        </data>
</root>
```

</details>

---

<details>
  <summary>
    <code>POST</code>
    <code><b>/updateConsumer</b></code>
  </summary>

##### Payload

```

<?xml version="1.0" encoding="UTF-8"?>
    <root>
        <data>
            <clientCode>O00001</clientCode> // Código do cliente
            <consumerCode>000000000031988156796</consumerCode> // Código do consumidor
            <name>Novo Consumidor</name> // Nome do consumidor
            <email></email> // Email do consumidor
            <phone>(31) 98815-6796</phone> // Telefone do consumidor
            <cpf></cpf> // CPF do consumidor
            <birthday></birthday> // Data de nascimento
            <gender></gender> // Gênero
            <cep></cep> // Cep do consumidor
            <street></street> // Rua do endereço do consumidor
            <neighborhood></neighborhood> // Bairro do endereço do consumidor
            <addressNumber></addressNumber> // Número da residencia
            <complement></complement> // Complemento do endereço
            <state></state> // Estado 
            <city></city> // Cidade
        </data>
    </root>

```

##### Response

```
<root>
    <error>false</error>
    <message>Registro atualizado com sucesso.</message>
    <data/>
</root>
```

</details>

---

<details>
  <summary>
    <code>POST</code>
    <code><b>/getSaleResume</b></code>
  </summary>

##### Payload

```
<?xml version="1.0" encoding="UTF-8"?>
    <root>
        <data>
            <subsidiaryCode>0107</subsidiaryCode> // Código da filial
            <posCode>002</posCode> // Código do caixa
            <clientCode>O00001</clientCode> // Código do cliente
            <consumerCode>000000000031988887777</consumerCode> // Código do consumidor
            <saleSeq>0000000002</saleSeq> // Sequencial da venda
            <posOpeningDate>10/01/2024 09:29:15</posOpeningDate> // Data de abertura do caixa
        </data>
    </root>
```

##### Response

```
<?xml version="1.0" encoding="UTF-8"?>
    <root>
        <error>false</error>
        <message></message>
        <data>
            <consumerName>MARCOS AUGUSTO REIS SOUZA</consumerName> // Nome do consumidor
            <consumerCode>000000000031988887777</consumerCode> // Código do consumidor
            <clientCode>O00001</clientCode> // Código do cliente
        <balance>
            <generatedBalance>1.28</generatedBalance> // Saldo gerado
            <redeemBalance>1.5</redeemBalance> // Saldo resgatado
            <totalBalance>1.2</totalBalance> // Saldo total
            <expiration>29/01/2024</expiration> // Data de validade do saldo
        </balance>
        </data>
    </root>
```

</details>


---

<details>
  <summary>
    <code>POST</code>
    <code><b>/registerConsumer</b></code>
  </summary>

##### Payload

```
<?xml version="1.0" encoding="UTF-8"?>
<root>
    <data>
        <name>consumidor teste site</name>
        <email></email>
        <phone>(31) 999998888</phone>
        <cpf>11111111111</cpf>
        <birthday>31-02-1965</birthday>
        <gender>Masculino</gender>
        <cep>33099999</cep>
        <street>Rua Teste</street>
        <neighborhood>Bairro Teste</neighborhood>
        <addressNumber>171</addressNumber>
        <complement></complement>
        <state>MG</state>
        <city>Belo Horizonte</city>
    </data>
</root>
```

##### Response

```
<?xml version="1.0" encoding="UTF-8"?>
<root>
    <error>false</error>
    <message>Registro atualizado com sucesso.</message>
    <data/>
</root>
```

</details>


---

<details>
  <summary>
    <code>POST</code>
    <code><b>/generateQrcode</b></code>
  </summary>

##### Payload

```
<root>
    <data>
        <subsidiaryCode>0001</subsidiaryCode>
        <clientCode>O00001</clientCode>
        <consumerCode>000000000000000000001</consumerCode>
        <phone>31999998888</phone>
    </data>
</root>
```

##### Response

```
<?xml version="1.0" encoding="UTF-8"?>
<root>
    <error>false</error>
    <message></message>
    <data>
        <clientCode>O00001</clientCode>
        <consumerCode>000000000000000000001</consumerCode>
        <url>sh.teknisa.com/b050e04f</url>
    </data>
</root>
```

</details>


---

<details>
  <summary>
    <code>POST</code>
    <code><b>/getClientLayout</b></code>
  </summary>

##### Response

```   
<root>
    <error>false</error>
    <message></message>
    <data>
        <primaryColor>0800ff</primaryColor>
        <secondaryColor>ff020a</secondaryColor>
        <buttonColor>00ff08</buttonColor>
        <footerColor>ffff02</footerColor>
        <mainLogo></mainLogo>
        <footerLogo></footerLogo>
        <successTxt>Texto de cadastro de sucesso de exemplo.</successTxt>
        <registeredTxt>Texto Usuário Já Cadastrado de exemplo</registeredTxt>
        <saleNotFoundTxt>Texto de Vendas não Encontradas de exemplo</saleNotFoundTxt>
        <hasOwnApp>1</hasOwnApp>
        <playStoreLink>https://teknisa.com</playStoreLink>
        <appStoreLink>https://teknisa.com</appStoreLink>
        <sendSms>N</sendSms>
    </data>
</root>
```

</details>

---

### ToDo

#### Campanhas Homologadas:

- [x] Saldo por % venda
- [ ] Produto
- [ ] Valor de venda - DOTZ
- [ ] Percentual por produto
- [ ] Desconto por venda

#### Rota getBalance:

- [x] Consultar saldo do consummidor na tavela MOVPONTOCONS
- [x] Retornar os dados do consumidor e saldo

#### Rota creditBalance:

- [x] Validar se o consumidor precisa ser inserido no Oracle
- [x] Validar se existe uma campanha saldo por % venda ativa
- [x] Calcular a pontuação gerada por item da venda
- [x] Inserir um único registro de crédito da VENDA na MOVPONTOCONS ( NRSEQVENDA deverá ter o valor null e gravar NRSEQVENDAMSDE)
- [x] Enviar SMS para o telefone de consumidor (Telefone informado no CDCONSUMIDOR, CDIDCONSUMID, NRTELECONS)
- [x] Retornar o saldo adquirido na venda e a data de validade dos pontos (Se aplicavél )

#### Rota redeemBalance:

- [x] Enviar os dados do resgate: Cód.Campanha, Cód.Cliente, Cód.Consumidor, valor de desconto utilizado na venda
- [x] Inserir registro de débito venda na MOVPONTOCONS (O saldo deverá ser debitado dos registros mais antigos para o mais recente)
- [x] SMS de débito de saldo

#### Rota Cancelamento de pedido:

- [x] Enviar os dados de cancelamento: Cód.Filial, Cód.Caixa, Cód.venda
- [x] O estorno deverá ser realizado no crédito válido mais antigo do consumidor
- [x] Se a venda gerou crédito de saldo, o saldo(VRSALDOCONS) deverá ser zerado no registro, para que não seja utilizado

#### Problemas Encontrados:

##### A Tabela MOVPONTOCONS precisa da venda.

- [x] ENVIAR A IDENTIFICAÇÃO DA VENDA NO MSDE PRA SER GRAVADA NA MOVPONTOCONS: CDFILIAL, CDCAIXA, NRSEQVENDAMSDE, DTABERCAIX.
- [x] CRIAR UM ÍNDICE SOBRE OS CAMPOS CDFILIAL, CDCAIXA, NRSEQVENDAMSDE, DTABERCAIX NA MOVPONTOCONS.
- [x] O FOSEXP VAI ATUALIZAR A CHAVE DA VENDA NA TABELA MOVPONTOCONS, COM BASE NOS CAMPOS CDFILIAL, CDCAIXA, NRSEQVENDAMSDE, DTABERCAIX.
