<?php

function getFullHeaders(){
    $allHeaders = getallheaders();
    if(!empty($allHeaders)) {
        $allHeaders = array_change_key_case($allHeaders);
        return $allHeaders;
    } else {
        return [];
    }
}

function getKey(String $key, $defaultValue = ""){
    $loweredKey = strtolower($key);
    $allHeaders = getFullHeaders();
    $responseValue = isset($allHeaders[$loweredKey]) ? $allHeaders[$loweredKey] : $defaultValue;
    return $responseValue;
}

function getConfigurationFile($fileType) {

    $extClientPreset = getValueFromEnv("EXTERNAL_CLIENT_PRESET");
    $environmentPath = "";

    $client     = getKey('x-client-env', null);
    $clientType = getKey('x-client-type', null);

    if(empty($extClientPreset) && (empty($client) || empty($clientType))) { 
        errorMessage("Invalid client preset");        
    }

    if($clientType == 'saas') {
        return json_encode([
            "driver"        => "oci8",
            "host"          => "",
            "port"          => "",
            "user"          => "/",
            "password"      => "",
            "dbname"        => $client,
            "charset"       => "AL32UTF8",
            "service"       => true,
            "sessionMode"   => "OCI_CRED_EXT"
        ]);   
    } else {
        $environmentPath = __DIR__ . "/../clients/external/$extClientPreset/$fileType";   
    }
    
    if(!file_exists($environmentPath)) {
        errorMessage("Invalid $fileType file");
    }
    
    return file_get_contents($environmentPath);
}


function getValueFromEnv($key, $value = "") {
    if(!file_exists(__DIR__ . "/../env.ini")) {
       errorMessage("Invalid .env file");
    }

    $envFile = fopen(__DIR__ . "/../env.ini", "r");
    while(!feof($envFile)) {
        $line         = fgets($envFile);
        $tmp          = explode('=', $line);
        $currentKey   = $tmp[0];
        $currentValue = trim($tmp[1]);
        if($currentKey == $key) {
            $value = $currentValue;
            break;                
        }
    }
    
    fclose($envFile);
    return $value;
}

function getClientName() {
    $extClientPreset = getValueFromEnv("EXTERNAL_CLIENT_PRESET");
    $clientDataPath = "";

    if(empty($extClientPreset)) {
        $clientDataPath  = __DIR__ . "/../configs/configs.json"; //SaaS Version
    } else {
        $clientDataPath = __DIR__ . "/../clients/external/$extClientPreset/config.json";   
    }

    if(!file_exists($clientDataPath)) {
        errorMessage("Invalid client config file");
    }
    
    return json_decode(file_get_contents($clientDataPath), true);
}

function getClientEnv() {
    $client = getKey('x-client-env', null);
    return $client;
}


function errorMessage($error) {
    echo json_encode([
        'error'   => true,
        'message' => $error,
        'data'    => []
    ]); 
    die;
}
