<?php
namespace App\Factory;

use App\Exception;

class IntegrationFactory {
    private $connection;
    private $util;

    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
                                     
       $this->connection = $connection;
       $this->util       = $util;
    }
    
    public function getBalance($clientCode, $consumerCode, $campaignCode) {
        $sql = "
            SELECT M.CDCAMPANHA, M.CDCLIENTE, M.CDCONSUMIDOR, O.NMCONSUMIDOR, SUM(M.VRSALDOCONS) AS VRSALDOCONS
              FROM MOVPONTOCONS M, CONSUMIDOR O
             WHERE M.CDCLIENTE    = :CDCLIENTE
               AND M.CDCONSUMIDOR = :CDCONSUMIDOR
               AND M.CDCAMPANHA   = :CDCAMPANHA
               AND M.IDTPMOVCONS  = 'C'
               AND M.CDCLIENTE    = O.CDCLIENTE
               AND M.CDCONSUMIDOR = O.CDCONSUMIDOR
               AND ((M.DTVALIDAPONTO >= :DIA) OR (M.DTVALIDAPONTO IS NULL))
            GROUP BY M.CDCAMPANHA, M.CDCLIENTE, M.CDCONSUMIDOR, O.NMCONSUMIDOR
        ";
        
        $params = [
            ':CDCLIENTE'    => $clientCode,
            ':CDCONSUMIDOR' => $consumerCode,
            ':CDCAMPANHA'   => $campaignCode,
            ':DIA'          => date('d/m/Y')
        ];
         
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getPercentValuePerProduct($subsidiaryCode, $campaignId, $product, $today) {
        $sql = "
            SELECT P.VRPONTOPROD
              FROM PRODUTO O, PRODCAMPANHA P, FILICAMPANHA F,
                   (SELECT MAX(O.CDARVPROD) AS CDARVPROD
                      FROM PRODCAMPANHA P, PRODUTO O, PRODUTO R
                     WHERE P.CDCAMPANHA = :CDCAMPANHA
                       AND P.CDPRODUTO  = O.CDPRODUTO
                       AND R.CDPRODUTO  = :CDPRODUTO
                       AND INSTR(R.CDARVPROD, O.CDARVPROD) = 1) A
             WHERE (P.CDCAMPANHA = :CDCAMPANHA)
               AND (F.CDFILIAL   = :CDFILIAL  )
               AND (P.CDCAMPANHA = F.CDCAMPANHA )
               AND (P.CDPRODUTO  = O.CDPRODUTO  )
               AND (O.CDARVPROD  = A.CDARVPROD  )
               AND (TO_DATE(:DTVENDA, 'DD/MM/YYYY') BETWEEN P.DTINIVIGPROD AND P.DTFIMVIGPROD)
        ";
        
        $params = [
            ':CDFILIAL'   => $subsidiaryCode,
            ':CDCAMPANHA' => $campaignId,
            ':CDPRODUTO'  => $product,
            ':DTVENDA'    => $today
        ];
        
        $result = $this->connection->fetchAssoc($sql, $params);
        
        return isset($result['VRPONTOPROD']) ?  $result['VRPONTOPROD'] : null;
    }
    
    public function getCampaigns($subsidiaryCode, $today) {
        $sql = "
            SELECT C.CDCAMPANHA, C.DSCAMPANHA, C.IDCAMPANHA, C.NRDIASVALPONTO, TO_CHAR(C.DTFIMVIGCAMP, 'DD-MM-YYYY') AS DTFIMVIGCAMP, 
                   C.IDUTILVOUCHER, C.VRPONTOVOUCHER, C.NRDIAVALVOUCHER, C.QTMAXPONTDIA,
                   C.VRPESOCAMP, C.VRPONTOCAMP
              FROM FILICAMPANHA F , CAMPANHA C 
            WHERE CDFILIAL =  :CDFILIAL
              AND F.CDCAMPANHA = C.CDCAMPANHA 
              AND (TO_DATE(:TODAY, 'DD/MM/YYYY') BETWEEN C.DTINIVIGCAMP AND C.DTFIMVIGCAMP)
        ";
        
        $params = [
            ':CDFILIAL' => $subsidiaryCode,
            ':TODAY'    => $today
        ];
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function insertConsumerScore($subsidiaryCode, $posCode, $clientCode, $consumerCode, $campaign, $nrSeqMovCons, $score, $expiration, $saleSeq, $dtAbertur, $obs) { 
        $sql = "
            INSERT INTO MOVPONTOCONS
               (CDCAMPANHA, CDCLIENTE, CDCONSUMIDOR, NRSEQMOVCONS, IDTPMOVCONS,
                VRMOVPONTOCONS, DTMOVPONTOCONS, CDFILIAL, CDCAIXA,
                 DTINCLUSAO, DSOBSMOVPTOCONS, VRSALDOCONS, DTVALIDAPONTO, NRSEQVENDAMSDE, DTABERTUR
                )
            VALUES
               (:CDCAMPANHA, :CDCLIENTE, :CDCONSUMIDOR, :NRSEQMOVCONS, :IDTPMOVCONS,
                :VRMOVPONTOCONS, TO_DATE(:DTMOVPONTOCONS, 'DD/MM/YYYY HH24:MI:SS'), :CDFILIAL, :CDCAIXA,
                 TO_DATE(:DTINCLUSAO, 'DD/MM/YYYY HH24:MI:SS'), :DSOBSMOVPTOCONS, :VRSALDOCONS, TO_DATE(:DTVALIDAPONTO, 'DD/MM/YYYY'),
                 :NRSEQVENDAMSDE, TO_DATE(:DTABERTUR, 'DD/MM/YYYY HH24:MI:SS')
                )
        ";
        
        $params = [
            ':CDCAMPANHA'      => $campaign['CDCAMPANHA'],
            ':CDCLIENTE'       => $clientCode,
            ':CDCONSUMIDOR'    => $consumerCode,
            ':NRSEQMOVCONS'    => $nrSeqMovCons,
            ':IDTPMOVCONS'     => 'C',
            ':VRMOVPONTOCONS'  => str_replace(".", ",", $score),
            ':DTMOVPONTOCONS'  => date('d/m/Y H:i:s'),
            ':CDFILIAL'        => $subsidiaryCode,
            ':CDCAIXA'         => $posCode,
            ':DTINCLUSAO'      => date('d/m/Y H:i:s'),
            ':DSOBSMOVPTOCONS' => $obs,
            ':VRSALDOCONS'     => str_replace(".", ",", $score),
            ':DTVALIDAPONTO'   => $expiration,
            ':NRSEQVENDAMSDE'  => $saleSeq,
            ':DTABERTUR'       => $dtAbertur
        ];

        $this->connection->executeQuery($sql, $params);
    }
    
    public function getCreditsInput($consumerDetails, $campaignCode, $today) {
        $sql = "
            SELECT M.CDCAMPANHA, M.CDCLIENTE, M.CDCONSUMIDOR, M.NRSEQMOVCONS,
                   M.IDTPMOVCONS, M.VRSALDOCONS, M.DTVALIDAPONTO, C.IDCAMPANHA
             FROM MOVPONTOCONS M, CAMPANHA C
                WHERE M.CDCAMPANHA = C.CDCAMPANHA
                 AND M.CDCAMPANHA = :CDCAMPANHA
                 AND M.CDCLIENTE = :CDCLIENTE
                 AND M.CDCONSUMIDOR = :CDCONSUMIDOR
                 AND M.IDTPMOVCONS = 'C'
                 AND ((M.DTVALIDAPONTO >= TO_DATE(:DIA, 'DD/MM/YYYY')) OR (M.DTVALIDAPONTO IS NULL)) 
            ORDER BY DTMOVPONTOCONS, DTVALIDAPONTO, NRSEQMOVCONS
        ";
        
        $params = [
            ':CDCAMPANHA'   => $campaignCode,
            ':CDCLIENTE'    => $consumerDetails['CDCLIENTE'],
            ':CDCONSUMIDOR' => $consumerDetails['CDCONSUMIDOR'],
            ':DIA'          => $today
        ];

        return $this->connection->fetchAll($sql, $params);
    }
    
    public function updateObsMovPontoCons($movement, $obsMovement) {
        $sql = "
            UPDATE MOVPONTOCONS
                SET DSOBSMOVPTOCONS  = :DSOBSMOVPTOCONS
            WHERE CDCAMPANHA = :CDCAMPANHA
             AND CDCLIENTE = :CDCLIENTE
             AND CDCONSUMIDOR = :CDCONSUMIDOR
             AND NRSEQMOVCONS = :NRSEQMOVCONS
        ";
        
        $params = [
            ':CDCAMPANHA'       => $movement['CDCAMPANHA'],
            ':CDCLIENTE'        => $movement['CDCLIENTE'],
            ':CDCONSUMIDOR'     => $movement['CDCONSUMIDOR'],
            ':NRSEQMOVCONS'     => $movement['NRSEQMOVCONS'],
            ':DSOBSMOVPTOCONS'  => $obsMovement
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function updateMovPontoCons($movement, $discount) {
        $sql = "
            UPDATE MOVPONTOCONS
                SET 
                    VRSALDOCONS = :VRSALDOCONS,
                    IDTPMOVCONS = :IDTPMOVCONS
            WHERE CDCAMPANHA = :CDCAMPANHA
             AND CDCLIENTE = :CDCLIENTE
             AND CDCONSUMIDOR = :CDCONSUMIDOR
             AND NRSEQMOVCONS = :NRSEQMOVCONS
        ";
        
        $params = [
            ':CDCAMPANHA'      => $movement['CDCAMPANHA'],
            ':CDCLIENTE'       => $movement['CDCLIENTE'],
            ':CDCONSUMIDOR'    => $movement['CDCONSUMIDOR'],
            ':NRSEQMOVCONS'    => $movement['NRSEQMOVCONS'],
            ':IDTPMOVCONS'     => $movement['IDTPMOVCONS'],
            ':VRSALDOCONS'     => str_replace(".", ",", $discount)
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function insertRedeemBalance($subsidiaryCode, $posCode, $clientCode, $consumerCode, $campaignCode, $nrSeqMovCons, $balanceUsed, $saleSeq, $dtAbertur) { 
        $sql = "
            INSERT INTO MOVPONTOCONS
               (CDCAMPANHA, CDCLIENTE, CDCONSUMIDOR, NRSEQMOVCONS, IDTPMOVCONS,
                VRMOVPONTOCONS, DTMOVPONTOCONS, CDFILIAL, CDCAIXA,
                 DTINCLUSAO, DSOBSMOVPTOCONS, VRSALDOCONS, DTVALIDAPONTO, NRSEQVENDAMSDE, DTABERTUR
                )
            VALUES
               (:CDCAMPANHA, :CDCLIENTE, :CDCONSUMIDOR, :NRSEQMOVCONS, :IDTPMOVCONS,
                :VRMOVPONTOCONS, TO_DATE(:DTMOVPONTOCONS, 'DD/MM/YYYY HH24:MI:SS'), :CDFILIAL, :CDCAIXA,
                TO_DATE(:DTINCLUSAO, 'DD/MM/YYYY HH24:MI:SS'), :DSOBSMOVPTOCONS, :VRSALDOCONS, TO_DATE(:DTVALIDAPONTO, 'DD/MM/YYYY'),
                :NRSEQVENDAMSDE, TO_DATE(:DTABERTUR, 'DD/MM/YYYY HH24:MI:SS')
                )
        ";
        
        $params = [
            ':CDCAMPANHA'      => $campaignCode,
            ':CDCLIENTE'       => $clientCode,
            ':CDCONSUMIDOR'    => $consumerCode,
            ':NRSEQMOVCONS'    => $nrSeqMovCons,
            ':IDTPMOVCONS'     => 'D',
            ':VRMOVPONTOCONS'  => str_replace(".", ",", $balanceUsed),
            ':DTMOVPONTOCONS'  => date('d/m/Y H:i:s'),
            ':CDFILIAL'        => $subsidiaryCode,
            ':CDCAIXA'         => $posCode,
            ':DTINCLUSAO'      => date('d/m/Y H:i:s'),
            ':DSOBSMOVPTOCONS' => 'Resgate de saldo por % venda. (fidelity-api)',
            ':VRSALDOCONS'     => 0,
            ':DTVALIDAPONTO'   => null,
            ':NRSEQVENDAMSDE'  => $saleSeq,
            ':DTABERTUR'       => $dtAbertur
        ];

        $this->connection->executeQuery($sql, $params);
    }
    
    public function getBranchDetails($subsidiaryCode) {
        $sql = "
            SELECT CDFILIAL, NMFILIAL
                FROM FILIAL
            WHERE CDFILIAL = :CDFILIAL
        ";
        
        $params = [
            ':CDFILIAL' => $subsidiaryCode
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getMovementConsumer($subsidiaryCode, $posCode, $consumerDetails, $saleSeq, $dtabertur) {
        $sql = "
            SELECT M.CDCAMPANHA, M.CDCLIENTE, M.CDCONSUMIDOR, M.NRSEQMOVCONS, M.VRMOVPONTOCONS,
                   M.IDTPMOVCONS, M.VRSALDOCONS, TO_CHAR(M.DTVALIDAPONTO, 'DD-MM-YYYY') AS DTVALIDAPONTO, C.IDCAMPANHA, C.NRDIASVALPONTO, 
                   TO_CHAR(M.DTABERTUR, 'DD/MM/YYYY HH24:MI:SS') AS DTABERTUR, TO_CHAR(C.DTFIMVIGCAMP, 'DD-MM-YYYY') AS DTFIMVIGCAMP
             FROM MOVPONTOCONS M, CAMPANHA C
                WHERE M.CDCAMPANHA = C.CDCAMPANHA
                 AND M.CDCLIENTE = :CDCLIENTE
                 AND M.CDCONSUMIDOR = :CDCONSUMIDOR
                 AND M.CDFILIAL = :CDFILIAL
                 AND M.CDCAIXA = :CDCAIXA
                 AND M.NRSEQVENDAMSDE = :NRSEQVENDAMSDE
                 AND M.DTABERTUR = TO_DATE(:DTABERTUR, 'DD/MM/YYYY HH24:MI:SS')
                 AND ((M.DTVALIDAPONTO >= TO_DATE(:DIA, 'DD/MM/YYYY')) OR (M.DTVALIDAPONTO IS NULL)) 
            ORDER BY DTMOVPONTOCONS, DTVALIDAPONTO, NRSEQMOVCONS
        ";
        
        $params = [
            ':CDCLIENTE'      => $consumerDetails['CDCLIENTE'],
            ':CDCONSUMIDOR'   => $consumerDetails['CDCONSUMIDOR'],
            ':CDFILIAL'       => $subsidiaryCode,
            ':CDCAIXA'        => $posCode,
            ':NRSEQVENDAMSDE' => $saleSeq,
            ':DTABERTUR'      => $dtabertur,
            ':DIA'            => date('d/m/Y')
        ];

        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getReturnMovement($movement) {
        $sql = "
            SELECT M.CDCAMPANHA, M.CDCLIENTE, M.CDCONSUMIDOR, M.NRSEQMOVCONS,
                   M.VRSALDOCONS, M.DTVALIDAPONTO
             FROM MOVPONTOCONS M
                 WHERE M.CDCAMPANHA = :CDCAMPANHA
                 AND M.CDCLIENTE = :CDCLIENTE
                 AND M.CDCONSUMIDOR = :CDCONSUMIDOR
                 AND M.IDTPMOVCONS = 'C'
                 AND ((M.DTVALIDAPONTO >= TO_DATE(:DIA, 'DD/MM/YYYY')) OR (M.DTVALIDAPONTO IS NULL))
                 AND ROWNUM = 1
            ORDER BY DTMOVPONTOCONS, DTVALIDAPONTO, NRSEQMOVCONS
        ";
        
        $params = [
            ':CDCAMPANHA'     => $movement['CDCAMPANHA'],
            ':CDCLIENTE'      => $movement['CDCLIENTE'],
            ':CDCONSUMIDOR'   => $movement['CDCONSUMIDOR'],
            ':DIA'            => date('d/m/Y')
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getClientLayout() {
        $sql = "
            SELECT CORPRIMARIA, CORSECUNDARIA, CORBOTAO, CORRODAPE, LOGOPRINCIPAL, LOGORODAPE, TEXTOSUCESSO, 
                   TEXTOJACADASTRADO, TEXTOVENDASNAOENCONTRADO, APPPROPRIO, LINKPLAYSTORE, LINKAPPSTORE, ENVIOSMS, 
                   URLSMS, TOKENTELE, NMLOJASMS, MENSASALDOSMS, MENSACREDGERA, MENSACADCONSUMIDOR
              FROM CONFIGSITECASHBACK
        ";
         
        return $this->connection->fetchAssoc($sql);
    }
}
